/////////////////////////////////////////////////////////////////////////////
//
//	MG35TOOL
//		This program will split MG35 firmware into it's separate component
//		parts, and join them back into a firmware image.
//
//	Author:		Jeff Curless
//
//	Information for this application was taken from:
//		http://mediagate.pbwiki.com/MG35Firmware
//
/////////////////////////////////////////////////////////////////////////////

#define MG35_TAG_STRING		"UPGIXM02"
#define MG35_BOOT_STRING	"BOOT"
#define MG35_ROFS_STRING	"ROFS"
#define MG35_BOOT_OFFSET	0
#define MG35_ROFS_OFFSET	0x40000 

#define MG350_TAG_STRING	"UPGIMG03"
#define MG350_BOOT_OFFSET	0x46000000
#define MG350_ROFS_OFFSET	0x46020000

#define MG25_TAG_STRING		"MG02"
#define MG25_BOOT_OFFSET	0
#define MG25_ROFS_OFFSET	0x40000 


//////////////////////////////////////////////////////////////////////////////
//
//	Handle big-endian/little-endian issues with DWORD defintions
//
//////////////////////////////////////////////////////////////////////////////

typedef unsigned char BYTE;
typedef unsigned int DWORD;

struct DWORD_LE {
	unsigned char b1;
	unsigned char b2;
	unsigned char b3;
	unsigned char b4;

	//
	// Auto convert to little endian
	//
	void operator=(DWORD item){ 
		b1 = item & 0xFF;
		b2 = (item>>8) & 0xFF;
		b3 = (item>>16) & 0xFF;
		b4 = (item>>24) & 0xFF;
	}

	//
	// Allow cast back to big endian
	//
	operator DWORD (){
		DWORD Value;

		Value = b1;
		Value |= (b2 << 8);
		Value |= (b3 << 16);
		Value |= (b4 << 24);
		return Value;
	}
				
};


//////////////////////////////////////////////////////////////////////////////
//
//	Command info structure
//
//////////////////////////////////////////////////////////////////////////////

struct CmdInfo{
	bool	bSplit;
	bool	bJoin;
	bool	bBoot;
	bool	bMG25;

	char	*BootImage;
	char	*KernelImage;
	char	*CramfsImage;
	char	*UpdateFile;
	char    *FirmwareTag;
	DWORD   Version;

	DWORD	BootSize;
	DWORD	KernelSize;
	DWORD	CramfsSize;
	DWORD	BootOffsetInFlash;
	DWORD	ROFSOffsetInFlash;

	CmdInfo::CmdInfo(){
		bSplit = false;
		bJoin  = false;
		bBoot  = false;
		bMG25  = false;

		BootImage    = "boot.img";
		KernelImage  = "linux.gz";
		CramfsImage  = "cramfs.img";
		Version      = 0x00010404;
		UpdateFile   = NULL;
		FirmwareTag  = MG35_TAG_STRING;
		BootOffsetInFlash = MG35_BOOT_OFFSET;
		ROFSOffsetInFlash = MG35_ROFS_OFFSET;
	};
};

typedef struct CmdInfo CInfo;
//////////////////////////////////////////////////////////////////////////////
//
//	UpgradeFirmware header, this header is the same for all variants
//
//////////////////////////////////////////////////////////////////////////////

struct UpgradeHeader {
	char	 Tag[8];
	DWORD_LE Version;
	char	 NextSection[4];
};

struct UpgradeHeader25 {					// MG25 only!
	char	 Tag[4];
	DWORD_LE Version;
	DWORD_LE FileSize;
	char	 NextSection[4];
};

//////////////////////////////////////////////////////////////////////////////
//
//	Boot firmware header.  BootSection starts at same offset as NextSection
//	in UpgradeHeader (i.e. they are the same field).
//
//////////////////////////////////////////////////////////////////////////////

struct BOOTHeader {
	char	 BootSection[4];
	DWORD_LE Checksum;
	DWORD_LE OffsetToBootLoaderInFlash;
	DWORD_LE Length;
	BYTE	 DataStart;	// Checksum start!
};

struct BOOTHeader25 {					// MG25 only!
	char	 BootSection[4];
	DWORD_LE OffsetToBootLoaderInFlash;
	DWORD_LE Length;
	DWORD_LE Checksum;
	BYTE	 DataStart;	// Checksum start!
};

//////////////////////////////////////////////////////////////////////////////
//
//	Rom file system, not really.. but it is the ROFS section, located after
//  the boot header, *or* directly after the main header, RofsSection overlaying
//  NextSection element in upgradeheader
//
//////////////////////////////////////////////////////////////////////////////

struct ROFSHeader {
	char	 RofsSection[4];
	DWORD_LE Checksum;
	DWORD_LE OffsetInFlash;
	DWORD_LE Length;
	DWORD_LE OffsetToCramfs;	// Checksum start!
	BYTE     DataStart;
};

struct ROFSHeader25 {					// MG25 only!
	char	 RofsSection[4];
	DWORD_LE OffsetInFlash;
	DWORD_LE Length;
	DWORD_LE Checksum;
	DWORD_LE OffsetToCramfs;	// Checksum start!
	BYTE     DataStart;
};


